@@ -23,6 +23,8 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include "file.h"
 
 #include <config.h>
@@ -61,13 +63,15 @@ same_inode(const KDE_struct_stat &src, const KDE_struct_stat &dest)
 
 extern int write_all(int fd, const char *buf, size_t len);
 
-void FileProtocol::copy( const KUrl &src, const KUrl &dest,
+void FileProtocol::copy( const KUrl &srcUrl, const KUrl &destUrl,
                          int _mode, JobFlags _flags )
 {
-    kDebug(7101) << "copy(): " << src << " -> " << dest << ", mode=" << _mode;
+    kDebug(7101) << "copy(): " << srcUrl << " -> " << destUrl << ", mode=" << _mode;
 
-    QByteArray _src( QFile::encodeName(src.toLocalFile()));
-    QByteArray _dest( QFile::encodeName(dest.toLocalFile()));
+    const QString src = srcUrl.toLocalFile();
+    const QString dest = destUrl.toLocalFile();
+    QByteArray _src( QFile::encodeName(src));
+    QByteArray _dest( QFile::encodeName(dest));
     KDE_struct_stat buff_src;
 #ifdef HAVE_POSIX_ACL
     acl_t acl;
@@ -75,18 +79,18 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
     if ( KDE_stat( _src.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, _src );
+           error(KIO::ERR_ACCESS_DENIED, src);
         else
-           error( KIO::ERR_DOES_NOT_EXIST, _src );
+           error(KIO::ERR_DOES_NOT_EXIST, src);
 	return;
     }
 
     if ( S_ISDIR( buff_src.st_mode ) ) {
-	error( KIO::ERR_IS_DIRECTORY, src.path() );
+	error(KIO::ERR_IS_DIRECTORY, src);
 	return;
     }
     if ( S_ISFIFO( buff_src.st_mode ) || S_ISSOCK ( buff_src.st_mode ) ) {
-	error( KIO::ERR_CANNOT_OPEN_FOR_READING, src.path() );
+	error(KIO::ERR_CANNOT_OPEN_FOR_READING, src);
 	return;
     }
 
@@ -96,19 +100,19 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     {
         if (S_ISDIR(buff_dest.st_mode))
         {
-           error( KIO::ERR_DIR_ALREADY_EXIST, _dest );
+           error(KIO::ERR_DIR_ALREADY_EXIST, dest);
            return;
         }
 
 	if ( same_inode( buff_dest, buff_src) )
 	{
-	    error( KIO::ERR_IDENTICAL_FILES, _dest );
+	    error(KIO::ERR_IDENTICAL_FILES, dest);
 	    return;
 	}
 
         if (!(_flags & KIO::Overwrite))
         {
-           error( KIO::ERR_FILE_ALREADY_EXIST, _dest );
+           error(KIO::ERR_FILE_ALREADY_EXIST, dest);
            return;
         }
 
@@ -124,7 +128,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
     int src_fd = KDE_open( _src.data(), O_RDONLY);
     if ( src_fd < 0 ) {
-	error( KIO::ERR_CANNOT_OPEN_FOR_READING, _src );
+	error(KIO::ERR_CANNOT_OPEN_FOR_READING, src);
 	return;
     }
 
@@ -141,11 +145,11 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
     int dest_fd = KDE_open(_dest.data(), O_CREAT | O_TRUNC | O_WRONLY, initialMode);
     if ( dest_fd < 0 ) {
-	kDebug(7101) << "###### COULD NOT WRITE " << dest.url();
+	kDebug(7101) << "###### COULD NOT WRITE " << dest;
         if ( errno == EACCES ) {
-            error( KIO::ERR_WRITE_ACCESS_DENIED, _dest );
+            error(KIO::ERR_WRITE_ACCESS_DENIED, dest);
         } else {
-            error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, _dest );
+            error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest);
         }
         ::close(src_fd);
         return;
@@ -196,17 +200,17 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
             kDebug(7101) << "sendfile() error:" << strerror(errno);
             if ( errno == ENOSPC ) // disk full
             {
-                error( KIO::ERR_DISK_FULL, _dest );
+                error(KIO::ERR_DISK_FULL, dest);
                 remove( _dest.data() );
             }
             else {
-                error( KIO::ERR_SLAVE_DEFINED,
-                        i18n("Cannot copy file from %1 to %2. (Errno: %3)",
-                          src.toLocalFile(), dest.toLocalFile(), errno ) );
+                error(KIO::ERR_SLAVE_DEFINED,
+                      i18n("Cannot copy file from %1 to %2. (Errno: %3)",
+                      src, dest, errno));
             }
           } else
 #endif
-          error( KIO::ERR_COULD_NOT_READ, _src );
+          error(KIO::ERR_COULD_NOT_READ, src);
           ::close(src_fd);
           ::close(dest_fd);
 #ifdef HAVE_POSIX_ACL
@@ -226,13 +230,13 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
            if ( errno == ENOSPC ) // disk full
            {
-              error( KIO::ERR_DISK_FULL, _dest );
+              error(KIO::ERR_DISK_FULL, dest);
               remove( _dest.data() );
            }
            else
            {
               kWarning(7101) << "Couldn't write[2]. Error:" << strerror(errno);
-              error( KIO::ERR_COULD_NOT_WRITE, _dest );
+              error(KIO::ERR_COULD_NOT_WRITE, dest);
            }
 #ifdef HAVE_POSIX_ACL
            if (acl) acl_free(acl);
@@ -251,7 +255,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     if (::close( dest_fd))
     {
         kWarning(7101) << "Error when closing file descriptor[2]:" << strerror(errno);
-        error( KIO::ERR_COULD_NOT_WRITE, _dest );
+        error(KIO::ERR_COULD_NOT_WRITE, dest);
 #ifdef HAVE_POSIX_ACL
         if (acl) acl_free(acl);
 #endif
@@ -267,10 +271,10 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 #endif
         )
        {
-           KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(_dest);
+           KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath(dest);
            // Eat the error if the filesystem apparently doesn't support chmod.
            if ( mp && mp->testFileSystemFlag( KMountPoint::SupportsChmod ) )
-               warning( i18n( "Could not change permissions for\n%1" ,  dest.toLocalFile() ) );
+               warning(i18n("Could not change permissions for\n%1", dest));
        }
     }
 #ifdef HAVE_POSIX_ACL
@@ -283,7 +287,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     ut.modtime = buff_src.st_mtime;
     if ( ::utime( _dest.data(), &ut ) != 0 )
     {
-        kWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg( _dest.data() );
+        kWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg(dest);
     }
 
     processedSize( buff_src.st_size );
@@ -292,7 +296,6 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
 void FileProtocol::listDir( const KUrl& url)
 {
-    kDebug(7101) << "========= LIST " << url.url() << " =========";
     if (!url.isLocalFile()) {
         KUrl redir(url);
 	redir.setProtocol(config()->readEntry("DefaultRemoteProtocol", "smb"));
@@ -301,98 +304,117 @@ void FileProtocol::listDir( const KUrl& url)
 	finished();
 	return;
     }
-    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
-    KDE_struct_stat buff;
-    if ( KDE_stat( _path.data(), &buff ) == -1 ) {
-	error( KIO::ERR_DOES_NOT_EXIST, _path );
-	return;
-    }
-
-    if ( !S_ISDIR( buff.st_mode ) ) {
-	error( KIO::ERR_IS_FILE, _path );
-	return;
-    }
-
-    DIR *dp = 0L;
-    KDE_struct_dirent *ep;
-
-    dp = opendir( _path.data() );
+    const QString path(url.toLocalFile());
+    const QByteArray _path(QFile::encodeName(path));
+    DIR* dp = opendir(_path.data());
     if ( dp == 0 ) {
-        switch (errno)
-        {
+        switch (errno) {
+        case ENOENT:
+            error(KIO::ERR_DOES_NOT_EXIST, path);
+            return;
+        case ENOTDIR:
+            error(KIO::ERR_IS_FILE, path);
+            break;
 #ifdef ENOMEDIUM
 	case ENOMEDIUM:
-            error( ERR_SLAVE_DEFINED,
-                   i18n( "No media in device for %1", url.toLocalFile() ) );
+            error(ERR_SLAVE_DEFINED,
+                  i18n("No media in device for %1", path));
             break;
-#else
-        case ENOENT: // just to avoid the warning
 #endif
         default:
-            error( KIO::ERR_CANNOT_ENTER_DIRECTORY, _path );
+            error(KIO::ERR_CANNOT_ENTER_DIRECTORY, path);
             break;
         }
 	return;
     }
 
+    const QString sDetails = metaData(QLatin1String("details"));
+    const int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
+    //kDebug(7101) << "========= LIST " << url << "details=" << details << " =========";
+    UDSEntry entry;
+
     // Don't make this a QStringList. The locale file name we get here
     // should be passed intact to createUDSEntry to avoid problems with
     // files where QFile::encodeName(QFile::decodeName(a)) != a.
     QList<QByteArray> entryNames;
-    while ( ( ep = KDE_readdir( dp ) ) != 0L )
-	entryNames.append( ep->d_name );
+    KDE_struct_dirent *ep;
+    if (details == 0) {
+        // Fast path (for recursive deletion, mostly)
+        // Simply emit the name and file type, nothing else.
+        while ( ( ep = KDE_readdir( dp ) ) != 0 ) {
+            entry.clear();
+            entry.insert(KIO::UDSEntry::UDS_NAME, QFile::decodeName(ep->d_name));
+            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
+                         (ep->d_type & DT_DIR) ? S_IFDIR : S_IFREG );
+            if (ep->d_type & DT_LNK) {
+                // for symlinks obey the UDSEntry contract and provide UDS_LINK_DEST
+                // even if we don't know the link dest (and DeleteJob doesn't care...)
+                entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QLatin1String("Dummy Link Target"));
+            }
+            listEntry(entry, false);
+        }
+        closedir( dp );
+        listEntry( entry, true ); // ready
+    } else {
+        while ( ( ep = KDE_readdir( dp ) ) != 0 ) {
+            entryNames.append( ep->d_name );
+        }
 
-    closedir( dp );
-    totalSize( entryNames.count() );
+        closedir( dp );
+        totalSize( entryNames.count() );
 
-    /* set the current dir to the path to speed up
-       in not having to pass an absolute path.
-       We restore the path later to get out of the
-       path - the kernel wouldn't unmount or delete
-       directories we keep as active directory. And
-       as the slave runs in the background, it's hard
-       to see for the user what the problem would be */
-    char path_buffer[PATH_MAX];
-    getcwd(path_buffer, PATH_MAX - 1);
-    if ( chdir( _path.data() ) )  {
-        if (errno == EACCES)
-            error(ERR_ACCESS_DENIED, _path);
-        else
-            error(ERR_CANNOT_ENTER_DIRECTORY, _path);
-        finished();
+        /* set the current dir to the path to speed up
+           in not having to pass an absolute path.
+           We restore the path later to get out of the
+           path - the kernel wouldn't unmount or delete
+           directories we keep as active directory. And
+           as the slave runs in the background, it's hard
+           to see for the user what the problem would be */
+        char path_buffer[PATH_MAX];
+        path_buffer[0] = '\0';
+        (void) getcwd(path_buffer, PATH_MAX - 1);
+        if ( chdir( _path.data() ) )  {
+            if (errno == EACCES)
+                error(ERR_ACCESS_DENIED, path);
+            else
+                error(ERR_CANNOT_ENTER_DIRECTORY, path);
+            finished();
+        }
+
+        QList<QByteArray>::ConstIterator it = entryNames.constBegin();
+        QList<QByteArray>::ConstIterator end = entryNames.constEnd();
+        for (; it != end; ++it) {
+            entry.clear();
+            if ( createUDSEntry( QFile::decodeName(*it),
+                                 *it /* we can use the filename as relative path*/,
+                                 entry, details, true ) )
+                listEntry( entry, false);
+        }
+
+        listEntry( entry, true ); // ready
+
+        //kDebug(7101) << "============= COMPLETED LIST ============";
+
+        if (*path_buffer)
+            chdir(path_buffer);
     }
-
-    UDSEntry entry;
-    QList<QByteArray>::ConstIterator it = entryNames.constBegin();
-    QList<QByteArray>::ConstIterator end = entryNames.constEnd();
-    for (; it != end; ++it) {
-        entry.clear();
-        if ( createUDSEntry( QFile::decodeName(*it),
-                             *it /* we can use the filename as relative path*/,
-                             entry, 2, true ) )
-          listEntry( entry, false);
-    }
-
-    listEntry( entry, true ); // ready
-
-    kDebug(7101) << "============= COMPLETED LIST ============";
-
-    chdir(path_buffer);
     finished();
 }
 
-void FileProtocol::rename( const KUrl &src, const KUrl &dest,
+void FileProtocol::rename( const KUrl &srcUrl, const KUrl &destUrl,
                            KIO::JobFlags _flags )
 {
     char off_t_should_be_64_bits[sizeof(off_t) >= 8 ? 1 : -1]; (void) off_t_should_be_64_bits;
-    QByteArray _src(QFile::encodeName(src.toLocalFile()));
-    QByteArray _dest(QFile::encodeName(dest.toLocalFile()));
+    const QString src = srcUrl.toLocalFile();
+    const QString dest = destUrl.toLocalFile();
+    const QByteArray _src(QFile::encodeName(src));
+    const QByteArray _dest(QFile::encodeName(dest));
     KDE_struct_stat buff_src;
     if ( KDE_lstat( _src.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, _src );
+           error(KIO::ERR_ACCESS_DENIED, src);
         else
-           error( KIO::ERR_DOES_NOT_EXIST, _src );
+           error(KIO::ERR_DOES_NOT_EXIST, src);
         return;
     }
 
@@ -404,19 +426,19 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
     {
         if (S_ISDIR(buff_dest.st_mode))
         {
-           error( KIO::ERR_DIR_ALREADY_EXIST, _dest );
+           error(KIO::ERR_DIR_ALREADY_EXIST, dest);
            return;
         }
 
 	if ( same_inode( buff_dest, buff_src) )
 	{
-	    error( KIO::ERR_IDENTICAL_FILES, _dest );
+	    error(KIO::ERR_IDENTICAL_FILES, dest);
 	    return;
 	}
 
         if (!(_flags & KIO::Overwrite))
         {
-           error( KIO::ERR_FILE_ALREADY_EXIST, _dest );
+           error(KIO::ERR_FILE_ALREADY_EXIST, dest);
            return;
         }
     }
@@ -424,16 +446,16 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
     if ( KDE_rename( _src.data(), _dest.data()))
     {
         if (( errno == EACCES ) || (errno == EPERM)) {
-            error( KIO::ERR_ACCESS_DENIED, _dest );
+            error(KIO::ERR_ACCESS_DENIED, dest);
         }
         else if (errno == EXDEV) {
-           error( KIO::ERR_UNSUPPORTED_ACTION, QLatin1String("rename"));
+           error(KIO::ERR_UNSUPPORTED_ACTION, QLatin1String("rename"));
         }
         else if (errno == EROFS) { // The file is on a read-only filesystem
-           error( KIO::ERR_CANNOT_DELETE, _src );
+           error(KIO::ERR_CANNOT_DELETE, src);
         }
         else {
-           error( KIO::ERR_CANNOT_RENAME, _src );
+           error(KIO::ERR_CANNOT_RENAME, src);
         }
         return;
     }
@@ -441,10 +463,11 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
     finished();
 }
 
-void FileProtocol::symlink( const QString &target, const KUrl &dest, KIO::JobFlags flags )
+void FileProtocol::symlink( const QString &target, const KUrl &destUrl, KIO::JobFlags flags )
 {
+    const QString dest = destUrl.toLocalFile();
     // Assume dest is local too (wouldn't be here otherwise)
-    if ( ::symlink( QFile::encodeName( target ), QFile::encodeName( dest.path() ) ) == -1 )
+    if ( ::symlink( QFile::encodeName(target), QFile::encodeName(dest) ) == -1 )
     {
         // Does the destination already exist ?
         if ( errno == EEXIST )
@@ -452,54 +475,53 @@ void FileProtocol::symlink( const QString &target, const KUrl &dest, KIO::JobFla
             if ( (flags & KIO::Overwrite) )
             {
                 // Try to delete the destination
-                if ( unlink( QFile::encodeName( dest.path() ) ) != 0 )
+                if ( unlink( QFile::encodeName(dest) ) != 0 )
                 {
-                    error( KIO::ERR_CANNOT_DELETE, dest.path() );
+                    error(KIO::ERR_CANNOT_DELETE, dest);
                     return;
                 }
                 // Try again - this won't loop forever since unlink succeeded
-                symlink( target, dest, flags );
+                symlink( target, destUrl, flags );
             }
             else
             {
                 KDE_struct_stat buff_dest;
-                KDE_lstat( QFile::encodeName( dest.path() ), &buff_dest );
+                KDE_lstat( QFile::encodeName(dest), &buff_dest );
                 if (S_ISDIR(buff_dest.st_mode))
-                    error( KIO::ERR_DIR_ALREADY_EXIST, dest.path() );
+                    error(KIO::ERR_DIR_ALREADY_EXIST, dest);
                 else
-                    error( KIO::ERR_FILE_ALREADY_EXIST, dest.path() );
+                    error(KIO::ERR_FILE_ALREADY_EXIST, dest);
                 return;
             }
         }
         else
         {
             // Some error occurred while we tried to symlink
-            error( KIO::ERR_CANNOT_SYMLINK, dest.path() );
+            error(KIO::ERR_CANNOT_SYMLINK, dest);
             return;
         }
     }
     finished();
 }
 
-void FileProtocol::del( const KUrl& url, bool isfile)
+void FileProtocol::del(const KUrl& url, bool isfile)
 {
-    QByteArray _path( QFile::encodeName(url.toLocalFile()));
+    const QString path = url.toLocalFile();
+    const QByteArray _path( QFile::encodeName(path));
     /*****
      * Delete files
      *****/
 
     if (isfile) {
-	kDebug( 7101 ) <<  "Deleting file "<< url.url();
-
-	// TODO deletingFile( source );
+	kDebug(7101) << "Deleting file "<< url;
 
 	if ( unlink( _path.data() ) == -1 ) {
             if ((errno == EACCES) || (errno == EPERM))
-               error( KIO::ERR_ACCESS_DENIED, _path );
+               error(KIO::ERR_ACCESS_DENIED, path);
             else if (errno == EISDIR)
-               error( KIO::ERR_IS_DIRECTORY, _path );
+               error(KIO::ERR_IS_DIRECTORY, path);
             else
-               error( KIO::ERR_CANNOT_DELETE, _path );
+               error(KIO::ERR_CANNOT_DELETE, path);
 	    return;
 	}
     } else {
@@ -509,13 +531,14 @@ void FileProtocol::del( const KUrl& url, bool isfile)
        *****/
 
       kDebug( 7101 ) << "Deleting directory " << url.url();
-
+      if (!deleteRecursive(path))
+          return;
       if ( ::rmdir( _path.data() ) == -1 ) {
 	if ((errno == EACCES) || (errno == EPERM))
-	  error( KIO::ERR_ACCESS_DENIED, _path );
+	  error(KIO::ERR_ACCESS_DENIED, path);
 	else {
 	  kDebug( 7101 ) << "could not rmdir " << perror;
-	  error( KIO::ERR_COULD_NOT_RMDIR, _path );
+	  error(KIO::ERR_COULD_NOT_RMDIR, path);
 	  return;
 	}
       }
@@ -526,7 +549,8 @@ void FileProtocol::del( const KUrl& url, bool isfile)
 
 void FileProtocol::chown( const KUrl& url, const QString& owner, const QString& group )
 {
-    const QByteArray _path( QFile::encodeName(url.toLocalFile()) );
+    const QString path = url.toLocalFile();
+    const QByteArray _path( QFile::encodeName(path) );
     uid_t uid;
     gid_t gid;
 
@@ -560,14 +584,56 @@ void FileProtocol::chown( const KUrl& url, const QString& owner, const QString&
         switch ( errno ) {
             case EPERM:
             case EACCES:
-                error( KIO::ERR_ACCESS_DENIED, _path );
+                error(KIO::ERR_ACCESS_DENIED, path);
                 break;
             case ENOSPC:
-                error( KIO::ERR_DISK_FULL, _path );
+                error(KIO::ERR_DISK_FULL, path);
                 break;
             default:
-                error( KIO::ERR_CANNOT_CHOWN, _path );
+                error(KIO::ERR_CANNOT_CHOWN, path);
         }
     } else
         finished();
 }
+
+void FileProtocol::stat( const KUrl & url )
+{
+    if (!url.isLocalFile()) {
+        KUrl redir(url);
+	redir.setProtocol(config()->readEntry("DefaultRemoteProtocol", "smb"));
+	redirection(redir);
+	kDebug(7101) << "redirecting to " << redir.url();
+	finished();
+	return;
+    }
+
+    /* directories may not have a slash at the end if
+     * we want to stat() them; it requires that we
+     * change into it .. which may not be allowed
+     * stat("/is/unaccessible")  -> rwx------
+     * stat("/is/unaccessible/") -> EPERM            H.Z.
+     * This is the reason for the -1
+     */
+    const QString path(url.path(KUrl::RemoveTrailingSlash));
+    const QByteArray _path( QFile::encodeName(path));
+    const QString sDetails = metaData(QLatin1String("details"));
+    const int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
+
+    UDSEntry entry;
+    if ( !createUDSEntry( url.fileName(), _path, entry, details, true /*with acls*/ ) )
+    {
+        error(KIO::ERR_DOES_NOT_EXIST, path);
+        return;
+    }
+#if 0
+///////// debug code
+    MetaData::iterator it1 = mOutgoingMetaData.begin();
+    for ( ; it1 != mOutgoingMetaData.end(); it1++ ) {
+        kDebug(7101) << it1.key() << " = " << it1.data();
+    }
+/////////
+#endif
+    statEntry( entry );
+
+    finished();
+}
